@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.56 2003/07/13 08:52:36 mpeseng Exp $
+ *  $Id: ephy-bookmarks.c,v 1.59 2003/08/24 18:20:58 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -210,8 +210,7 @@ ephy_bookmarks_autocompletion_source_init (EphyAutocompletionSourceIface *iface)
 static void
 ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 {
-	int i, id;
-	EphyNode *node;
+	int i;
 
 	for (i = 0; i < n_default_topics; i++)
 	{
@@ -220,13 +219,12 @@ ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 
 	for (i = 0; i < n_default_bookmarks; i++)
 	{
-		ephy_bookmarks_add (eb, default_bookmarks[i].title,
-				    default_bookmarks[i].location);
+		EphyNode *bmk;
 
-		node = ephy_bookmarks_find_bookmark (eb, default_bookmarks[i].location);
-		if (node == NULL) break;
-		id = ephy_node_get_id (node);
-		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE, id);
+		bmk = ephy_bookmarks_add (eb, default_bookmarks[i].title,
+				          default_bookmarks[i].location);
+		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE,
+						  ephy_node_get_id (bmk));
 	}
 }
 
@@ -326,17 +324,19 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 	g_return_val_if_fail (doc != NULL, FALSE);
 
 	root = xmlDocGetRootElement (doc);
-
+	child = root->children;
+	
 	tmp = xmlGetProp (root, "version");
 	if (tmp != NULL && strcmp (tmp, EPHY_BOOKMARKS_XML_VERSION) != 0)
 	{
 		g_warning ("Old bookmarks database format detected");
+
 		child = NULL;
 		result = FALSE;
 	}
 	g_free (tmp);
 
-	for (child = root->children; child != NULL; child = child->next)
+	for (; child != NULL; child = child->next)
 	{
 		EphyNode *node;
 
@@ -668,6 +668,8 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	GValue value = { 0, };
 	EphyNodeDb *db;
 
+	LOG ("Init");
+
         eb->priv = g_new0 (EphyBookmarksPrivate, 1);
 	eb->priv->toolbars_model = NULL;
 
@@ -884,13 +886,15 @@ ephy_bookmarks_set_address (EphyBookmarks *eb,
 	update_has_smart_address (bookmark, address);
 }
 
-EphyNode*
+EphyNode *
 ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 			      const char *url)
 {
 	GPtrArray *children;
 	int i;
 
+	g_return_val_if_fail (IS_EPHY_BOOKMARKS (eb), NULL);
+	g_return_val_if_fail (eb->priv->bookmarks != NULL, NULL);
 	g_return_val_if_fail (url != NULL, NULL);
 
 	children = ephy_node_get_children (eb->priv->bookmarks);
@@ -902,6 +906,7 @@ ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 		kid = g_ptr_array_index (children, i);
 		location = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_LOCATION);
+
 		if (location != NULL && strcmp (url, location) == 0)
 		{
 			ephy_node_thaw (eb->priv->bookmarks);
@@ -1114,13 +1119,44 @@ ephy_bookmarks_add_keyword (EphyBookmarks *eb,
 	return key;
 }
 
-void 
+void
 ephy_bookmarks_remove_keyword (EphyBookmarks *eb,
-				    EphyNode *keyword)
+			       EphyNode *keyword)
 {
 	ephy_node_remove_child (eb->priv->keywords, keyword);
 }
 
+char *
+ephy_bookmarks_get_topic_uri (EphyBookmarks *eb,
+			      EphyNode *node)
+{
+	char *uri;
+
+	if (ephy_bookmarks_get_bookmarks (eb) == node)
+	{
+		uri = g_strdup ("topic://Special/All");
+	}
+	else if (ephy_bookmarks_get_not_categorized (eb) == node)
+	{
+		uri = g_strdup ("topic://Special/NotCategorized");
+	}
+	else if (ephy_bookmarks_get_favorites (eb) == node)
+	{
+		uri = g_strdup ("topic://Special/Favorites");
+	}
+	else
+	{
+		const char *name;
+
+		name = ephy_node_get_property_string
+			(node, EPHY_NODE_KEYWORD_PROP_NAME);
+
+		uri = g_strdup_printf ("topic://%s", name);
+	}
+
+	return uri;
+}
+
 EphyNode *
 ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 			     const char *name,
@@ -1129,9 +1165,11 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 	EphyNode *node;
 	GPtrArray *children;
 	int i;
+	const char *topic_name;
 
 	g_return_val_if_fail (name != NULL, NULL);
 
+	topic_name = name;
 
 	if (g_utf8_strlen (name, -1) == 0)
 	{
@@ -1139,6 +1177,23 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 		return NULL;
 	}
 
+	if (strcmp (name, "topic://Special/All") == 0)
+	{
+		return ephy_bookmarks_get_bookmarks (eb);
+	}
+	else if (strcmp (name, "topic://Special/NotCategorized") == 0)
+	{
+		return ephy_bookmarks_get_not_categorized (eb);
+	}
+	else if (strcmp (name, "topic://Special/Favorites") == 0)
+	{
+		return ephy_bookmarks_get_favorites (eb);
+	}
+	else if (g_str_has_prefix (name, "topic://"))
+	{
+		topic_name += strlen ("topic://");
+	}
+
 	children = ephy_node_get_children (eb->priv->keywords);
 	node = NULL;
 	for (i = 0; i < children->len; i++)
@@ -1149,8 +1204,8 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 		 kid = g_ptr_array_index (children, i);
 		 key = ephy_node_get_property_string (kid, EPHY_NODE_KEYWORD_PROP_NAME);
 
-		 if ((partial_match && g_str_has_prefix (key, name) > 0) ||
-		     (!partial_match && strcmp (key, name) == 0))
+		 if ((partial_match && g_str_has_prefix (key, topic_name) > 0) ||
+		     (!partial_match && strcmp (key, topic_name) == 0))
 		 {
 			 node = kid;
 		 }
